@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (c) 2015-2024 Google, Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (c) 2015-2025 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,17 +25,15 @@
 #include <vulkan/utility/vk_format_utils.h>
 
 #include "generated/vk_extension_helper.h"
-#include "utils/vk_layer_utils.h"
 #include "layer_validation_tests.h"
+#include "vk_layer_config.h"
+#include "shader_helper.h"
+#include "containers/container_utils.h"
 
 #if defined(VK_USE_PLATFORM_METAL_EXT)
 #include "apple_wsi.h"
 #endif
 
-using std::string;
-using std::strncmp;
-using std::vector;
-
 template <typename C, typename F>
 typename C::iterator RemoveIf(C &container, F &&fn) {
     return container.erase(std::remove_if(container.begin(), container.end(), std::forward<F>(fn)), container.end());
@@ -44,7 +42,6 @@ typename C::iterator RemoveIf(C &container, F &&fn) {
 VkRenderFramework::VkRenderFramework()
     : instance_(nullptr),
       m_device(nullptr),
-      m_commandBuffer(nullptr),
       m_renderPass(VK_NULL_HANDLE),
       m_vertex_buffer(nullptr),
       m_width(256),   // default window width
@@ -60,19 +57,34 @@ VkRenderFramework::VkRenderFramework()
     m_clear_color.float32[1] = 0.25f;
     m_clear_color.float32[2] = 0.25f;
     m_clear_color.float32[3] = 0.0f;
+
+    // While it seems very undeal to have to allocate these on the heap, the alternative is to include vk_extension_helper.h (which
+    // contains custom_container.h) in the header which will add a measured 150ms second per test file to compile. This is the only
+    // code that requires custom hash maps
+    m_instance_extensions = new InstanceExtensions();
+    m_device_extensions = new DeviceExtensions();
 }
 
 VkRenderFramework::~VkRenderFramework() {
     ShutdownFramework();
     m_errorMonitor->Finish();
+
+    if (m_device_extensions) {
+        delete m_device_extensions;
+        m_device_extensions = nullptr;
+    }
+    if (m_instance_extensions) {
+        delete m_instance_extensions;
+        m_instance_extensions = nullptr;
+    }
 }
 
-VkPhysicalDevice VkRenderFramework::gpu() const {
+VkPhysicalDevice VkRenderFramework::Gpu() const {
     EXPECT_NE((VkInstance)0, instance_);  // Invalid to request gpu before instance exists
     return gpu_;
 }
 
-const VkPhysicalDeviceProperties &VkRenderFramework::physDevProps() const {
+const VkPhysicalDeviceProperties &VkRenderFramework::PhysicalDeviceProps() const {
     EXPECT_NE((VkPhysicalDevice)0, gpu_);  // Invalid to request physical device properties before gpu
     return physDevProps_;
 }
@@ -124,9 +136,9 @@ bool VkRenderFramework::DeviceExtensionSupported(const char *extension_name, con
 
     const auto enabled_layers = instance_layers_;  // assumes instance_layers_ contains enabled layers
 
-    auto extensions = device_obj.extensions();
+    auto extensions = device_obj.Extensions();
     for (const auto &layer : enabled_layers) {
-        const auto layer_extensions = device_obj.extensions(layer);
+        const auto layer_extensions = device_obj.Extensions(layer);
         extensions.insert(extensions.end(), layer_extensions.begin(), layer_extensions.end());
     }
 
@@ -180,13 +192,6 @@ void VkRenderFramework::InitFramework(void *instance_pnext) {
         }
     };
 
-    static bool driver_printed = false;
-    static bool print_driver_info = GetEnvironment("VK_LAYER_TESTS_PRINT_DRIVER") != "";
-    if (print_driver_info && !driver_printed &&
-        InstanceExtensionSupported(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME)) {
-        m_instance_extension_names.push_back(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    }
-
     // Beginning with the 1.3.216 Vulkan SDK, the VK_KHR_PORTABILITY_subset extension is mandatory.
 #ifdef VK_USE_PLATFORM_METAL_EXT
     AddRequiredExtensions(VK_KHR_PORTABILITY_ENUMERATION_EXTENSION_NAME);
@@ -274,17 +279,35 @@ void VkRenderFramework::InitFramework(void *instance_pnext) {
 
     m_errorMonitor->CreateCallback(instance_);
 
-    if (print_driver_info && !driver_printed) {
-        VkPhysicalDeviceDriverProperties driver_properties = vku::InitStructHelper();
-        VkPhysicalDeviceProperties2 physical_device_properties2 = vku::InitStructHelper(&driver_properties);
-        vk::GetPhysicalDeviceProperties2(gpu_, &physical_device_properties2);
-        printf("Driver Name = %s\n", driver_properties.driverName);
-        printf("Driver Info = %s\n", driver_properties.driverInfo);
+    static bool driver_printed = false;
+    static bool print_driver_info = GetEnvironment("VK_LAYER_TESTS_PRINT_DRIVER") != "";
 
-        driver_printed = true;
+    if (print_driver_info && !driver_printed) {
+        bool phys_dev_props_2_enabled = m_target_api_version.Minor() >= 1;
+        if (!phys_dev_props_2_enabled) {
+            for (const char *ext : vvl::make_span(ici.ppEnabledExtensionNames, ici.enabledExtensionCount)) {
+                if (strcmp(ext, VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME) == 0) {
+                    phys_dev_props_2_enabled = true;
+                    break;
+                }
+            }
+        }
+
+        if (InstanceExtensionSupported(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME) && phys_dev_props_2_enabled) {
+            VkPhysicalDeviceDriverProperties driver_properties = vku::InitStructHelper();
+            VkPhysicalDeviceProperties2 physical_device_properties2 = vku::InitStructHelper(&driver_properties);
+            vk::GetPhysicalDeviceProperties2(gpu_, &physical_device_properties2);
+            printf("Driver Name = %s\n", driver_properties.driverName);
+            printf("Driver Info = %s\n", driver_properties.driverInfo);
+
+            driver_printed = true;
+        } else {
+            printf(
+                "Could not print driver info - VK_KHR_get_physical_device_properties2 is either not supported or not enabled.\n");
+        }
     }
 
-    APIVersion used_version = std::min(m_instance_api_version, APIVersion(physDevProps_.apiVersion));
+    const APIVersion used_version = std::min(m_instance_api_version, APIVersion(physDevProps_.apiVersion));
     if (used_version < m_target_api_version) {
         GTEST_SKIP() << "At least Vulkan version 1." << m_target_api_version.Minor() << " is required";
     }
@@ -350,11 +373,12 @@ std::string VkRenderFramework::RequiredExtensionsNotSupported() const {
 }
 
 void VkRenderFramework::AddRequiredFeature(vkt::Feature feature) {
-    feature_requirements_.AddRequiredFeature(m_target_api_version, feature);
+    required_features_.AddRequiredFeature(m_target_api_version, feature);
+    features_to_enable_.AddRequiredFeature(m_target_api_version, feature);
 }
 
-void VkRenderFramework::AddDisabledFeature(vkt::Feature feature) {
-    feature_requirements_.AddDisabledFeature(m_target_api_version, feature);
+void VkRenderFramework::AddOptionalFeature(vkt::Feature feature) {
+    features_to_enable_.AddOptionalFeature(m_target_api_version, feature);
 }
 
 bool VkRenderFramework::AddRequestedInstanceExtensions(const char *ext_name) {
@@ -362,10 +386,10 @@ bool VkRenderFramework::AddRequestedInstanceExtensions(const char *ext_name) {
         return true;
     }
 
-    const auto &instance_exts_map = InstanceExtensions::GetInfoMap();
     bool is_instance_ext = false;
     vvl::Extension extension = GetExtension(ext_name);
-    if (instance_exts_map.find(extension) != instance_exts_map.cend()) {
+    const auto &instance_info = m_instance_extensions->GetInfo(extension);
+    if (instance_info.state) {
         if (!InstanceExtensionSupported(ext_name)) {
             return false;
         } else {
@@ -376,8 +400,7 @@ bool VkRenderFramework::AddRequestedInstanceExtensions(const char *ext_name) {
     // Different tables need to be used for extension dependency lookup depending on whether `ext_name` refers to a device or
     // instance extension
     if (is_instance_ext) {
-        const auto &info = InstanceExtensions::GetInfo(extension);
-        for (const auto &req : info.requirements) {
+        for (const auto &req : instance_info.requirements) {
             if (0 == strncmp(req.name, "VK_VERSION", 10)) {
                 continue;
             }
@@ -387,7 +410,7 @@ bool VkRenderFramework::AddRequestedInstanceExtensions(const char *ext_name) {
         }
         m_instance_extension_names.push_back(ext_name);
     } else {
-        const auto &info = DeviceExtensions::GetInfo(extension);
+        const auto &info = m_device_extensions->GetInfo(extension);
         for (const auto &req : info.requirements) {
             if (!AddRequestedInstanceExtensions(req.name)) {
                 return false;
@@ -433,18 +456,18 @@ bool VkRenderFramework::AddRequestedDeviceExtensions(const char *dev_ext_name) {
 
     // If this is an instance extension, just return true under the assumption instance extensions do not depend on any device
     // extensions.
-    const auto &instance_exts_map = InstanceExtensions::GetInfoMap();
     vvl::Extension extension = GetExtension(dev_ext_name);
-    if (instance_exts_map.find(extension) != instance_exts_map.cend()) {
+    const auto &instance_info = m_instance_extensions->GetInfo(extension);
+    if (instance_info.state) {
         return true;
     }
 
-    if (!DeviceExtensionSupported(gpu(), nullptr, dev_ext_name)) {
+    if (!DeviceExtensionSupported(Gpu(), nullptr, dev_ext_name)) {
         return false;
     }
     m_device_extension_names.push_back(dev_ext_name);
 
-    const auto &info = DeviceExtensions::GetInfo(extension);
+    const auto &info = m_device_extensions->GetInfo(extension);
     for (const auto &req : info.requirements) {
         if (!AddRequestedDeviceExtensions(req.name)) {
             return false;
@@ -454,7 +477,7 @@ bool VkRenderFramework::AddRequestedDeviceExtensions(const char *dev_ext_name) {
 }
 
 bool VkRenderFramework::IsPromotedDeviceExtension(const char *dev_ext_name) const {
-    auto device_version = std::min(m_target_api_version, APIVersion(physDevProps().apiVersion));
+    auto device_version = std::min(m_target_api_version, APIVersion(PhysicalDeviceProps().apiVersion));
     if (!device_version.Valid()) return false;
 
     const auto promotion_info_map = GetDevicePromotionInfoMap();
@@ -486,7 +509,6 @@ void VkRenderFramework::ShutdownFramework() {
     }
 
     m_command_buffer.destroy();
-    m_commandBuffer = nullptr;
     m_command_pool.destroy();
 
     if (m_second_queue) {
@@ -516,8 +538,8 @@ void VkRenderFramework::ShutdownFramework() {
 
     m_errorMonitor->DestroyCallback(instance_);
 
-    DestroySurface(m_surface);
-    DestroySurfaceContext(m_surface_context);
+    m_surface.Destroy();
+    m_surface_context.Destroy();
 
     vk::DestroyInstance(instance_, nullptr);
     instance_ = NULL;  // In case we want to re-initialize
@@ -527,7 +549,7 @@ void VkRenderFramework::ShutdownFramework() {
 ErrorMonitor &VkRenderFramework::Monitor() { return monitor_; }
 
 void VkRenderFramework::GetPhysicalDeviceFeatures(VkPhysicalDeviceFeatures *features) {
-    vk::GetPhysicalDeviceFeatures(gpu(), features);
+    vk::GetPhysicalDeviceFeatures(Gpu(), features);
 }
 
 // static
@@ -543,7 +565,7 @@ bool VkRenderFramework::IsPlatformMockICD() {
     if (VkRenderFramework::IgnoreDisableChecks()) {
         return false;
     } else {
-        return 0 == mock_icd_device_name.compare(physDevProps().deviceName);
+        return 0 == mock_icd_device_name.compare(PhysicalDeviceProps().deviceName);
     }
 }
 
@@ -614,35 +636,35 @@ void VkRenderFramework::InitState(VkPhysicalDeviceFeatures *features, void *crea
 
     // Apply required features after we are done with handling promoted extensions
     if (!features) {
-        if (feature_requirements_.HasFeatures2()) {
+        if (required_features_.HasFeatures2()) {
             if (vk::GetPhysicalDeviceFeatures2KHR) {
-                vk::GetPhysicalDeviceFeatures2KHR(gpu(), feature_requirements_.GetFeatures2());
+                vk::GetPhysicalDeviceFeatures2KHR(Gpu(), required_features_.GetFeatures2());
             } else {
-                vk::GetPhysicalDeviceFeatures2(gpu(), feature_requirements_.GetFeatures2());
+                vk::GetPhysicalDeviceFeatures2(Gpu(), required_features_.GetFeatures2());
             }
         } else {
-            GetPhysicalDeviceFeatures(feature_requirements_.GetFeatures());
+            GetPhysicalDeviceFeatures(required_features_.GetFeatures());
         }
 
-        if (const char *f = feature_requirements_.AnyRequiredFeatureDisabled()) {
+        if (const char *f = required_features_.AnyRequiredFeatureDisabled()) {
             GTEST_SKIP() << "Required feature " << f << " is not available on device, skipping test";
         }
 
-        feature_requirements_.EnforceDisableFeatures();
+        features_to_enable_.EnforceRequiredFeatures();
 
-        if (feature_requirements_.HasFeatures2()) {
+        if (features_to_enable_.HasFeatures2()) {
             if (create_device_pnext) {
                 // Chain to the end of the list
                 VkBaseOutStructure *p = reinterpret_cast<VkBaseOutStructure *>(create_device_pnext);
                 while (p->pNext != nullptr) {
                     p = p->pNext;
                 }
-                p->pNext = reinterpret_cast<VkBaseOutStructure *>(feature_requirements_.GetFeatures2());
+                p->pNext = reinterpret_cast<VkBaseOutStructure *>(features_to_enable_.GetFeatures2());
             } else {
-                create_device_pnext = feature_requirements_.GetFeatures2();
+                create_device_pnext = features_to_enable_.GetFeatures2();
             }
         } else {
-            features = feature_requirements_.GetFeatures();
+            features = features_to_enable_.GetFeatures();
         }
     }
 
@@ -652,30 +674,27 @@ void VkRenderFramework::InitState(VkPhysicalDeviceFeatures *features, void *crea
         vk::InitDeviceExtension(instance_, *m_device, device_ext_name);
     }
 
-    m_default_queue = m_device->QueuesWithGraphicsCapability()[0];
-
-    m_second_queue = [this]() -> vkt::Queue * {
-        if (m_device->QueuesWithGraphicsCapability().size() > 1) {
-            return m_device->QueuesWithGraphicsCapability()[1];  // skip default queue
+    std::vector<vkt::Queue *> queues;
+    vvl::Append(queues, m_device->QueuesWithGraphicsCapability());
+    for (vkt::Queue *queue_with_compute_caps : m_device->QueuesWithComputeCapability()) {
+        if (!vvl::Contains(queues, queue_with_compute_caps)) {
+            queues.emplace_back(queue_with_compute_caps);
         }
-        const auto &with_compute_caps = m_device->QueuesWithComputeCapability();
-        if (with_compute_caps.size() > 0 && with_compute_caps[0] != m_default_queue) {
-            return with_compute_caps[0];
+    }
+    for (vkt::Queue *queue_with_transfer_caps : m_device->QueuesWithTransferCapability()) {
+        if (!vvl::Contains(queues, queue_with_transfer_caps)) {
+            queues.emplace_back(queue_with_transfer_caps);
         }
-        if (with_compute_caps.size() > 1) {
-            return with_compute_caps[1];
-        }
-        const auto &with_transfer_caps = m_device->QueuesWithTransferCapability();
-        if (with_transfer_caps.size() > 0 && with_transfer_caps[0] != m_default_queue) {
-            return with_transfer_caps[0];
-        }
-        if (with_transfer_caps.size() > 1) {
-            return with_transfer_caps[1];
-        }
-        return nullptr;
-    }();
-    if (m_second_queue) {
-        m_second_queue_caps = m_device->phy().queue_properties_[m_second_queue->family_index].queueFlags;
+    }
+    m_default_queue = queues[0];
+    m_default_queue_caps = m_device->Physical().queue_properties_[m_default_queue->family_index].queueFlags;
+    if (queues.size() > 1) {
+        m_second_queue = queues[1];
+        m_second_queue_caps = m_device->Physical().queue_properties_[m_second_queue->family_index].queueFlags;
+    }
+    if (queues.size() > 2) {
+        m_third_queue = queues[2];
+        m_third_queue_caps = m_device->Physical().queue_properties_[m_third_queue->family_index].queueFlags;
     }
 
     m_depthStencil = new vkt::Image();
@@ -684,7 +703,6 @@ void VkRenderFramework::InitState(VkPhysicalDeviceFeatures *features, void *crea
 
     m_command_pool.Init(*m_device, m_device->graphics_queue_node_index_, flags);
     m_command_buffer.Init(*m_device, m_command_pool);
-    m_commandBuffer = &m_command_buffer;
 
     if (m_second_queue) {
         m_second_command_pool.Init(*m_device, m_second_queue->family_index, flags);
@@ -693,22 +711,27 @@ void VkRenderFramework::InitState(VkPhysicalDeviceFeatures *features, void *crea
 }
 
 void VkRenderFramework::InitSurface() {
-    // NOTE: Currently InitSurface can leak the WIN32 handle if called multiple times without first calling DestroySurfaceContext.
+    // NOTE: Currently InitSurface can leak the WIN32 handle if called multiple times without first calling Destroy() on
+    // m_surface_context.
     // This is intentional. Each swapchain/surface combo needs a unique HWND.
     VkResult result = CreateSurface(m_surface_context, m_surface);
     if (result != VK_SUCCESS) {
         GTEST_SKIP() << "Failed to create surface.";
     }
-    ASSERT_TRUE(m_surface != VK_NULL_HANDLE);
+    ASSERT_TRUE(m_surface.Handle() != VK_NULL_HANDLE);
 }
 
 #ifdef VK_USE_PLATFORM_WIN32_KHR
 LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
     return DefWindowProc(hwnd, uMsg, wParam, lParam);
 }
+
+void SurfaceContext::Resize(uint32_t width, uint32_t height) {
+    ::SetWindowPos(m_win32Window, NULL, 0, 0, (int)width, (int)height, SWP_NOMOVE);
+}
 #endif  // VK_USE_PLATFORM_WIN32_KHR
 
-VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSurfaceKHR &surface, VkInstance custom_instance) {
+VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, vkt::Surface &surface, VkInstance custom_instance) {
     const VkInstance surface_instance = (custom_instance != VK_NULL_HANDLE) ? custom_instance : instance();
     (void)surface_instance;
 #if defined(VK_USE_PLATFORM_WIN32_KHR)
@@ -720,13 +743,14 @@ VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSur
         wc.hInstance = window_instance;
         wc.lpszClassName = class_name;
         RegisterClass(&wc);
-        HWND window = CreateWindowEx(0, class_name, 0, 0, 0, 0, (int)m_width, (int)m_height, NULL, NULL, window_instance, NULL);
+        HWND window = CreateWindowEx(0, class_name, nullptr, 0, 0, 0, (int)m_width, (int)m_height, NULL, NULL, window_instance, NULL);
+        surface_context.m_win32Window = window;
         ShowWindow(window, SW_HIDE);
 
         VkWin32SurfaceCreateInfoKHR surface_create_info = vku::InitStructHelper();
         surface_create_info.hinstance = window_instance;
         surface_create_info.hwnd = window;
-        return vk::CreateWin32SurfaceKHR(surface_instance, &surface_create_info, nullptr, &surface);
+        return surface.Init(surface_instance, surface_create_info);
     }
 #endif
 
@@ -734,7 +758,7 @@ VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSur
     if (IsExtensionsEnabled(VK_EXT_METAL_SURFACE_EXTENSION_NAME)) {
         const VkMetalSurfaceCreateInfoEXT surface_create_info = vkt::CreateMetalSurfaceInfoEXT();
         assert(surface_create_info.pLayer != nullptr);
-        return vk::CreateMetalSurfaceEXT(surface_instance, &surface_create_info, nullptr, &surface);
+        return surface.Init(surface_instance, surface_create_info);
     }
 #endif
 
@@ -742,7 +766,7 @@ VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSur
     if (IsExtensionsEnabled(VK_KHR_ANDROID_SURFACE_EXTENSION_NAME)) {
         VkAndroidSurfaceCreateInfoKHR surface_create_info = vku::InitStructHelper();
         surface_create_info.window = VkTestFramework::window;
-        return vk::CreateAndroidSurfaceKHR(surface_instance, &surface_create_info, nullptr, &surface);
+        return surface.Init(surface_instance, surface_create_info);
     }
 #endif
 
@@ -757,7 +781,7 @@ VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSur
             VkXlibSurfaceCreateInfoKHR surface_create_info = vku::InitStructHelper();
             surface_create_info.dpy = surface_context.m_surface_dpy;
             surface_create_info.window = surface_context.m_surface_window;
-            return vk::CreateXlibSurfaceKHR(surface_instance, &surface_create_info, nullptr, &surface);
+            return surface.Init(surface_instance, surface_create_info);
         }
     }
 #endif
@@ -771,7 +795,7 @@ VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSur
             VkXcbSurfaceCreateInfoKHR surface_create_info = vku::InitStructHelper();
             surface_create_info.connection = surface_context.m_surface_xcb_conn;
             surface_create_info.window = window;
-            return vk::CreateXcbSurfaceKHR(surface_instance, &surface_create_info, nullptr, &surface);
+            return surface.Init(surface_instance, surface_create_info);
         }
     }
 #endif
@@ -779,52 +803,41 @@ VkResult VkRenderFramework::CreateSurface(SurfaceContext &surface_context, VkSur
     return VK_ERROR_UNKNOWN;
 }
 
-void VkRenderFramework::DestroySurface() {
-    DestroySurface(m_surface);
-    m_surface = VK_NULL_HANDLE;
-    DestroySurfaceContext(m_surface_context);
-    m_surface_context = {};
-}
-
-void VkRenderFramework::DestroySurface(VkSurfaceKHR &surface) {
-    if (surface != VK_NULL_HANDLE) {
-        vk::DestroySurfaceKHR(instance(), surface, nullptr);
-    }
-}
 #if defined(VK_USE_PLATFORM_XLIB_KHR)
 int IgnoreXErrors(Display *, XErrorEvent *) { return 0; }
 #endif
 
-void VkRenderFramework::DestroySurfaceContext(SurfaceContext &surface_context) {
+void SurfaceContext::Destroy() {
 #if defined(VK_USE_PLATFORM_WIN32_KHR)
-    if (surface_context.m_win32Window != nullptr) {
-        DestroyWindow(surface_context.m_win32Window);
+    if (m_win32Window != nullptr) {
+        DestroyWindow(m_win32Window);
+        m_win32Window = nullptr;
     }
 #endif
 
 #if defined(VK_USE_PLATFORM_XLIB_KHR)
-    if (surface_context.m_surface_dpy != nullptr) {
+    if (m_surface_dpy != nullptr) {
         // Ignore BadDrawable errors we seem to get during shutdown.
         // The default error handler will exit() and end the test suite.
         XSetErrorHandler(IgnoreXErrors);
-        XDestroyWindow(surface_context.m_surface_dpy, surface_context.m_surface_window);
-        surface_context.m_surface_window = None;
-        XCloseDisplay(surface_context.m_surface_dpy);
-        surface_context.m_surface_dpy = nullptr;
+        XDestroyWindow(m_surface_dpy, m_surface_window);
+        m_surface_window = None;
+        XCloseDisplay(m_surface_dpy);
+        m_surface_dpy = nullptr;
         XSetErrorHandler(nullptr);
     }
 #endif
 #if defined(VK_USE_PLATFORM_XCB_KHR)
-    if (surface_context.m_surface_xcb_conn != nullptr) {
-        xcb_disconnect(surface_context.m_surface_xcb_conn);
-        surface_context.m_surface_xcb_conn = nullptr;
+    if (m_surface_xcb_conn != nullptr) {
+        xcb_disconnect(m_surface_xcb_conn);
+        m_surface_xcb_conn = nullptr;
     }
 #endif
 }
 
 // Queries the info needed to create a swapchain and assigns it to the member variables of VkRenderFramework
 void VkRenderFramework::InitSwapchainInfo() {
-    auto info = GetSwapchainInfo(m_surface);
+    auto info = GetSwapchainInfo(m_surface.Handle());
     m_surface_capabilities = info.surface_capabilities;
     m_surface_formats = info.surface_formats;
     m_surface_present_modes = info.surface_present_modes;
@@ -835,7 +848,7 @@ void VkRenderFramework::InitSwapchainInfo() {
 // Makes query to get information about swapchain needed to create a valid swapchain object each test creating a swapchain will
 // need
 SurfaceInformation VkRenderFramework::GetSwapchainInfo(const VkSurfaceKHR surface) {
-    const VkPhysicalDevice physicalDevice = gpu();
+    const VkPhysicalDevice physicalDevice = Gpu();
 
     assert(surface != VK_NULL_HANDLE);
 
@@ -880,17 +893,17 @@ SurfaceInformation VkRenderFramework::GetSwapchainInfo(const VkSurfaceKHR surfac
 
 void VkRenderFramework::InitSwapchain(VkImageUsageFlags imageUsage, VkSurfaceTransformFlagBitsKHR preTransform) {
     RETURN_IF_SKIP(InitSurface());
-    ASSERT_TRUE(CreateSwapchain(m_surface, imageUsage, preTransform, m_swapchain));
+    m_swapchain = CreateSwapchain(m_surface.Handle(), imageUsage, preTransform);
+    ASSERT_TRUE(m_swapchain.initialized());
 }
 
-bool VkRenderFramework::CreateSwapchain(VkSurfaceKHR &surface, VkImageUsageFlags imageUsage,
-                                        VkSurfaceTransformFlagBitsKHR preTransform, VkSwapchainKHR &swapchain,
-                                        VkSwapchainKHR oldSwapchain) {
+vkt::Swapchain VkRenderFramework::CreateSwapchain(VkSurfaceKHR surface, VkImageUsageFlags imageUsage,
+                                                  VkSurfaceTransformFlagBitsKHR preTransform, VkSwapchainKHR oldSwapchain) {
     VkBool32 supported;
-    vk::GetPhysicalDeviceSurfaceSupportKHR(gpu(), m_device->graphics_queue_node_index_, surface, &supported);
+    vk::GetPhysicalDeviceSurfaceSupportKHR(Gpu(), m_device->graphics_queue_node_index_, surface, &supported);
     if (!supported) {
         // Graphics queue does not support present
-        return false;
+        return vkt::Swapchain{};
     }
 
     SurfaceInformation info = GetSwapchainInfo(surface);
@@ -898,7 +911,7 @@ bool VkRenderFramework::CreateSwapchain(VkSurfaceKHR &surface, VkImageUsageFlags
     // If this is being called from InitSwapchain, we need to also initialize all the VkRenderFramework
     // data associated with the swapchain since many tests use those variables. We can do this by checking
     // if the surface parameters address is the same as VkRenderFramework::m_surface
-    if (&surface == &m_surface) {
+    if (surface == m_surface.Handle()) {
         InitSwapchainInfo();
     }
 
@@ -907,8 +920,7 @@ bool VkRenderFramework::CreateSwapchain(VkSurfaceKHR &surface, VkImageUsageFlags
     swapchain_create_info.minImageCount = info.surface_capabilities.minImageCount;
     swapchain_create_info.imageFormat = info.surface_formats[0].format;
     swapchain_create_info.imageColorSpace = info.surface_formats[0].colorSpace;
-    swapchain_create_info.imageExtent = {info.surface_capabilities.minImageExtent.width,
-                                         info.surface_capabilities.minImageExtent.height};
+    swapchain_create_info.imageExtent = info.surface_capabilities.minImageExtent;
     swapchain_create_info.imageArrayLayers = 1;
     swapchain_create_info.imageUsage = imageUsage;
     swapchain_create_info.imageSharingMode = VK_SHARING_MODE_EXCLUSIVE;
@@ -918,29 +930,31 @@ bool VkRenderFramework::CreateSwapchain(VkSurfaceKHR &surface, VkImageUsageFlags
     swapchain_create_info.clipped = VK_FALSE;
     swapchain_create_info.oldSwapchain = oldSwapchain;
 
-    VkResult result = vk::CreateSwapchainKHR(device(), &swapchain_create_info, nullptr, &swapchain);
-    return result == VK_SUCCESS;
-}
-
-std::vector<VkImage> VkRenderFramework::GetSwapchainImages(const VkSwapchainKHR swapchain) {
-    uint32_t imageCount = 0;
-    vk::GetSwapchainImagesKHR(device(), swapchain, &imageCount, nullptr);
-    vector<VkImage> swapchainImages;
-    swapchainImages.resize(imageCount);
-    vk::GetSwapchainImagesKHR(device(), swapchain, &imageCount, swapchainImages.data());
-    return swapchainImages;
+    vkt::Swapchain swapchain(*m_device, swapchain_create_info);
+    return swapchain;
 }
 
 void VkRenderFramework::DestroySwapchain() {
     if (m_device && m_device->handle() != VK_NULL_HANDLE) {
         m_device->Wait();
-        if (m_swapchain != VK_NULL_HANDLE) {
-            vk::DestroySwapchainKHR(device(), m_swapchain, nullptr);
-            m_swapchain = VK_NULL_HANDLE;
+        if (m_swapchain.initialized()) {
+            m_swapchain.destroy();
         }
     }
 }
 
+void VkRenderFramework::SupportMultiSwapchain() {
+#ifdef VK_USE_PLATFORM_ANDROID_KHR
+    GTEST_SKIP() << "Android currently doesn't support multiple swapchain on all devices";
+#endif  // VK_USE_PLATFORM_ANDROID_KHR
+}
+
+void VkRenderFramework::SupportSurfaceResize() {
+    if (!SurfaceContext::CanResize()) {
+        GTEST_SKIP() << "VVL test framework does not support surface resizing on the current platform";
+    }
+}
+
 void VkRenderFramework::InitRenderTarget() { InitRenderTarget(1); }
 
 void VkRenderFramework::InitRenderTarget(uint32_t targets) { InitRenderTarget(targets, NULL); }
@@ -948,8 +962,8 @@ void VkRenderFramework::InitRenderTarget(uint32_t targets) { InitRenderTarget(ta
 void VkRenderFramework::InitRenderTarget(const VkImageView *dsBinding) { InitRenderTarget(1, dsBinding); }
 
 void VkRenderFramework::InitRenderTarget(uint32_t targets, const VkImageView *dsBinding) {
-    vector<VkAttachmentReference> color_references;
-    vector<VkAttachmentDescription> attachment_descriptions;
+    std::vector<VkAttachmentReference> color_references;
+    std::vector<VkAttachmentDescription> attachment_descriptions;
 
     attachment_descriptions.reserve(targets + 1);  // +1 for dsBinding
     color_references.reserve(targets);
@@ -982,7 +996,7 @@ void VkRenderFramework::InitRenderTarget(uint32_t targets, const VkImageView *ds
         m_renderPassClearValues.push_back(clear);
 
         VkFormatProperties props;
-        vk::GetPhysicalDeviceFormatProperties(m_device->phy().handle(), m_render_target_fmt, &props);
+        vk::GetPhysicalDeviceFormatProperties(m_device->Physical().handle(), m_render_target_fmt, &props);
 
         VkImageTiling tiling = VK_IMAGE_TILING_OPTIMAL;
         if (props.linearTilingFeatures & VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT) {
@@ -1100,13 +1114,18 @@ void VkRenderFramework::DestroyRenderTarget() {
 void VkRenderFramework::SetDefaultDynamicStatesExclude(const std::vector<VkDynamicState> &exclude, bool tessellation,
                                                        VkCommandBuffer commandBuffer) {
     const auto excluded = [&exclude](VkDynamicState state) {
-        return std::find(exclude.begin(), exclude.end(), state) != exclude.end();
+        for (const auto &check_state : exclude) {
+            if (check_state == state) {
+                return true;
+            }
+        }
+        return false;
     };
     if (!m_vertex_buffer) {
         m_vertex_buffer = new vkt::Buffer(*m_device, 32u, VK_BUFFER_USAGE_VERTEX_BUFFER_BIT, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
     }
 
-    VkCommandBuffer cmdBuffer = commandBuffer ? commandBuffer : m_commandBuffer->handle();
+    VkCommandBuffer cmdBuffer = commandBuffer ? commandBuffer : m_command_buffer.handle();
     VkViewport viewport = {0, 0, static_cast<float>(m_width), static_cast<float>(m_height), 0.0f, 1.0f};
     VkRect2D scissor = {{0, 0}, {m_width, m_height}};
     if (!excluded(VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT)) vk::CmdSetViewportWithCountEXT(cmdBuffer, 1u, &viewport);
@@ -1141,7 +1160,7 @@ void VkRenderFramework::SetDefaultDynamicStatesExclude(const std::vector<VkDynam
     if (!excluded(VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE)) vk::CmdSetStencilTestEnableEXT(cmdBuffer, VK_FALSE);
     if (!excluded(VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE)) vk::CmdSetDepthBiasEnableEXT(cmdBuffer, VK_FALSE);
     if (!excluded(VK_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE)) vk::CmdSetPrimitiveRestartEnableEXT(cmdBuffer, VK_FALSE);
-    if (!excluded(VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE_EXT)) vk::CmdSetRasterizerDiscardEnableEXT(cmdBuffer, VK_FALSE);
+    if (!excluded(VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE)) vk::CmdSetRasterizerDiscardEnableEXT(cmdBuffer, VK_FALSE);
     if (!excluded(VK_DYNAMIC_STATE_VERTEX_INPUT_EXT)) vk::CmdSetVertexInputEXT(cmdBuffer, 0u, nullptr, 0u, nullptr);
     if (!excluded(VK_DYNAMIC_STATE_LOGIC_OP_EXT)) vk::CmdSetLogicOpEXT(cmdBuffer, VK_LOGIC_OP_COPY);
     if (!excluded(VK_DYNAMIC_STATE_PATCH_CONTROL_POINTS_EXT)) vk::CmdSetPatchControlPointsEXT(cmdBuffer, 4u);
@@ -1216,4 +1235,10 @@ void VkRenderFramework::SetDefaultDynamicStatesAll(VkCommandBuffer cmdBuffer) {
     VkColorComponentFlags colorWriteMask =
         VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
     vk::CmdSetColorWriteMaskEXT(cmdBuffer, 0u, 1u, &colorWriteMask);
-}
\ No newline at end of file
+}
+
+std::vector<uint32_t> VkRenderFramework::GLSLToSPV(VkShaderStageFlagBits stage, const char *code, const spv_target_env env) {
+    std::vector<uint32_t> spv;
+    GLSLtoSPV(m_device->Physical().limits_, stage, code, spv, env);
+    return spv;
+}
